@@ -23,7 +23,7 @@
 
 #include "udunits2.h"
 
-static const char*  xmlPath;
+static const char*      xmlPath;
 static ut_system*	unitSystem;
 static ut_unit*		kilogram;
 static ut_unit*		meter;
@@ -322,9 +322,13 @@ test_utToString(void)
     CU_ASSERT_PTR_NOT_NULL(unit);
     CU_ASSERT(ut_format(unit, buf, sizeof(buf), asciiSymbolDef) != -1);
     CU_ASSERT_STRING_EQUAL(buf, string);
+    if (strcmp(buf, string))
+        printf("%d: buf=%s\n", __LINE__, buf);
 
     n = ut_format(unit, buf, 1, asciiSymbolDef);
     CU_ASSERT_EQUAL(n, strlen(string));
+    if (n != strlen(string))
+        printf("%d: n=%d\n", __LINE__, n);
     ut_free(unit);
 }
 
@@ -1460,6 +1464,8 @@ test_utOffsetByTime(void)
     CU_ASSERT_TRUE_FATAL(nchar < sizeof(buf));
     buf[nchar] = 0;
     CU_ASSERT_STRING_EQUAL(buf, "s @ 19700101T000000.0000000 UTC");
+    if (strcmp(buf, "s @ 19700101T000000.0000000 UTC"))
+        fprintf(stderr, "buf=%s\n", buf);
 
     minutesSinceTheMillenium =
         ut_offset_by_time(minute, ut_encode_time(2001, 1, 1, 0, 0, 0));
@@ -1596,13 +1602,13 @@ test_utOffsetByTime(void)
         }
 
         unit = ut_parse(unitSystem,
-                "second since 1970-01-01 0 +1", UT_ASCII);
+                "second since 1970-01-01 0 -1", UT_ASCII);
         CU_ASSERT_PTR_NOT_NULL(unit);
         if (unit) {
             converter = ut_get_converter(unit, secondsSinceTheEpoch);
             CU_ASSERT_PTR_NOT_NULL(converter);
             if (converter) {
-                CU_ASSERT_TRUE(areCloseDoubles(cv_convert_double(converter, 0), -3600));
+                CU_ASSERT_TRUE(areCloseDoubles(cv_convert_double(converter, 0), 3600));
                 cv_free(converter);
             }
             ut_free(unit);
@@ -1638,8 +1644,22 @@ test_ut_decode_time(void)
     CU_ASSERT_EQUAL(minute1, minute2);
     CU_ASSERT_EQUAL(second1, second2);
     CU_ASSERT_EQUAL(resolution1, resolution2);
-}
 
+    /*
+     * User reported getting 60 seconds back from ut_decode_time() for a time
+     * value of -3240.0.
+     */
+    //timeval = ut_encode_time(2000, 12, 31, 23, 5, 60);
+    timeval = -3240.0;
+    ut_decode_time(timeval, &year2, &month2, &day2, &hour2, &minute2,
+        &second2, &resolution2);
+    CU_ASSERT_EQUAL(year2, 2000);
+    CU_ASSERT_EQUAL(month2, 12);
+    CU_ASSERT_EQUAL(day2, 31);
+    CU_ASSERT_EQUAL(hour2, 23);
+    CU_ASSERT_EQUAL(minute2, 6);
+    CU_ASSERT_EQUAL(second2, 0);
+}
 
 
 static void
@@ -2232,9 +2252,37 @@ test_xml(void)
     ut_free_system(xmlSystem);
 }
 
+static void
+test_mm2_day2_divide(void)
+{
+    ut_system*  xmlSystem;
+    ut_unit*    unit1;
+    ut_unit*    unit2;
+    const char* spec = "mm2 day-2";
+
+    ut_set_error_message_handler(ut_write_to_stderr);
+    xmlSystem = ut_read_xml(xmlPath);
+    CU_ASSERT_PTR_NOT_NULL_FATAL(xmlSystem);
+
+    unit1 = ut_parse(xmlSystem, spec, UT_ASCII);
+    CU_ASSERT_PTR_NOT_NULL(unit1);
+    unit2 = ut_divide(unit1, unit1);
+
+    const char* expect = "1";
+    char        actual[128];
+    ut_format(unit2, actual, 128, UT_ASCII);
+    CU_ASSERT_STRING_EQUAL(actual, expect);
+    if (strcmp(actual, expect))
+        fprintf(stderr, "(%s)/(%s) = \"%s\"\n", spec, spec, actual);
+
+    ut_free(unit1);
+    ut_free(unit2);
+    ut_free_system(xmlSystem);
+}
+
 int
 main(
-    const int		    argc,
+    const int           argc,
     const char* const*	argv)
 {
     int	exitCode = EXIT_FAILURE;
@@ -2247,6 +2295,7 @@ main(
 	CU_Suite*	testSuite = CU_add_suite(__FILE__, setup, teardown);
 
 	if (testSuite != NULL) {
+	    CU_ADD_TEST(testSuite, test_mm2_day2_divide);
 	    CU_ADD_TEST(testSuite, test_unitSystem);
 	    CU_ADD_TEST(testSuite, test_utNewBaseUnit);
 	    CU_ADD_TEST(testSuite, test_utNewDimensionlessUnit);
