@@ -62,6 +62,7 @@
 #include <search.h>
 #endif
 
+#include <stdbool.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -202,6 +203,15 @@ union ut_unit {
 #define IS_LOG(unit)		((unit)->common.type == LOG)
 #define IS_TIMESTAMP(unit)	((unit)->common.type == TIMESTAMP)
 
+static bool areAlmostEqual(
+        double x,
+        double y)
+{
+    return (x == 0 || y == 0)
+	? fabs(x - y) < 10*DBL_EPSILON
+	: fabs(1.0 - x / y) < 10*DBL_EPSILON;
+}
+
 /*
  * The following function are declared here because they are used in the
  * basic-unit section  before they are defined in the product-unit section.
@@ -336,7 +346,7 @@ gregorianDateToJulianDay(year, month, day)
     if (jy >= 0)
     {
 	julday += 365 * jy;
-	julday += 0.25 * jy;
+	julday += (long)(0.25 * jy);
     }
     else
     {
@@ -397,48 +407,29 @@ ut_encode_clock(
     return (hours*60 + minutes)*60 + seconds;
 }
 
-
-/*
- * Decompose a value into a set of values accounting for uncertainty.
- */
-static void
-decompose(value, uncer, nbasis, basis, count)
-    double        value;
-    double        uncer;		/* >= 0 */
-    int           nbasis;
-    const double *basis;		/* all values > 0 */
-    double       *count;
+static int
+mydiv(  const double   numer,
+        const unsigned denom,
+        double* const  rem)
 {
-    int		i;
+    int n = (int)(abs(numer)/denom);
+    if (numer < 0)
+        n = -n;
+    *rem = numer - (long)n * (long)denom;
+    return n;
+}
 
-    for (i = 0; i < nbasis; i++)
-    {
-	double	r = fmod(value, basis[i]);	/* remainder */
-
-	/* Adjust remainder to minimum magnitude. */
-	if (ABS(2*r) > basis[i])
-	    r += r > 0
-		    ? -basis[i]
-		    :  basis[i];
-
-	if (ABS(r) <= uncer)
-	{
-	    /* The value equals a basis multiple within the uncertainty. */
-	    double	half = value < 0 ? -basis[i]/2 : basis[i]/2;
-	    modf((value+half)/basis[i], count+i);
-	    break;
-	}
-
-	value = basis[i] * modf(value/basis[i], count+i);
-    }
-
-    if (i >= nbasis) {
-	count[--i] += value;
-    }
-    else {
-	for (i++; i < nbasis; i++)
-	    count[i] = 0;
-    }
+static void
+decomp( double        value,
+        int* const    days,
+        int* const    hours,
+        int* const    minutes,
+        double* const seconds)
+{
+    double rem;
+    *days = mydiv(value, 86400, &rem);
+    *hours = mydiv(rem, 3600, &rem);
+    *minutes = mydiv(rem, 60, seconds);
 }
 
 
@@ -516,40 +507,19 @@ ut_decode_time(
     double	*second,
     double	*resolution)
 {
-    int			days;
-    int			hours;
-    int			minutes;
-    double		seconds;
-    double		uncer;		/* uncertainty of input value */
-    typedef union
-    {
-	double	    vec[7];
-	struct
-	{
-	    double	days;
-	    double	hours12;
-	    double	hours;
-	    double	minutes10;
-	    double	minutes;
-	    double	seconds10;
-	    double	seconds;
-	}	    ind;
-    } Basis;
-    Basis		counts;
-    static const Basis	basis = {86400, 43200, 3600, 600, 60, 10, 1};
+    int     days;
+    int     hours;
+    int     minutes;
+    int     d;
+    double  seconds;
+    /* Uncertainty of input value */
+    double  uncer = ldexp(value < 0 ? -value : value, -DBL_MANT_DIG);
 
-    uncer = ldexp(value < 0 ? -value : value, -DBL_MANT_DIG);
-
-    days = (int)floor(value/basis.ind.days);
-    value -= days * basis.ind.days;		/* make positive excess */
-
-    decompose(value, uncer, (int)(sizeof(basis.vec)/sizeof(basis.vec[0])),
-	   basis.vec, counts.vec);
-
-    days += counts.ind.days;
-    hours = (int)counts.ind.hours12 * 12 + (int)counts.ind.hours;
-    minutes = (int)counts.ind.minutes10 * 10 + (int)counts.ind.minutes;
-    seconds = (int)counts.ind.seconds10 * 10 + counts.ind.seconds;
+    days = (int)floor(value/86400.0);
+    // `long long` is necessary for dates like `1-01-01`
+    value -= days * (long long)86400; /* make positive excess */
+    decomp(value, &d, &hours, &minutes, &seconds);
+    days += d;
 
     if (seconds >= 60) {
 	seconds -= 60;
@@ -1633,7 +1603,7 @@ galileanNew(
 	unit = unit->galilean.unit;
     }
 
-    if (scale == 1 && offset == 0) {
+    if (areAlmostEqual(scale, 1) && areAlmostEqual(offset, 0)) {
 	newUnit = CLONE(unit);
     }
     else {
@@ -1791,8 +1761,8 @@ galileanMultiply(
 	}
     }
     else if (IS_GALILEAN(unit2)) {
-	const GalileanUnit* const	galilean2 = &unit2->galilean;
-	ut_unit*				tmp =
+	const GalileanUnit* const galilean2 = &unit2->galilean;
+	ut_unit*	          tmp =
 	    MULTIPLY(galilean1->unit, galilean2->unit);
 
 	if (tmp != NULL) {
