@@ -42,7 +42,7 @@
 #include "udunits2.h"
 
 #ifndef _XOPEN_PATH_MAX
-#   define _XOPEN_PATH_MAX 1024
+#   define _XOPEN_PATH_MAX 1024 // Includes terminating NUL
 #endif
 
 #define NAME_SIZE 128
@@ -97,7 +97,7 @@ static ut_status readXml(
 static File*            currFile = NULL;
 static ut_system*	unitSystem = NULL;
 static char*            text = NULL;
-static size_t           nbytes = 0;
+static size_t           nbytes = 0; /// Number of characters excluding NUL
 
 
 /*
@@ -117,7 +117,7 @@ ut_form_plural(
     const char*	plural = NULL;		/* failure */
 
     if (singular != NULL) {
-        int     length = strlen(singular);
+        size_t length = strlen(singular);
 
 	if (length + 3 >= sizeof(buf)) {
             ut_set_status(UT_SYNTAX);
@@ -1800,7 +1800,13 @@ endImport(
         path = text;
     }
     else {
-        (void)snprintf(buf, sizeof(buf), "%s/%s",
+        (void)snprintf(buf, sizeof(buf),
+#ifdef _MSC_VER
+            // The directory pathname has a trailing backslash on Windows
+            "%s%s",
+#else
+            "%s/%s",
+#endif
             XML_GetBase(currFile->parser), text);
 
         buf[sizeof(buf)-1] = 0;
@@ -2070,21 +2076,24 @@ readXml(
         ut_handle_error_message("Couldn't create XML parser");
     }
     else {
-        char        base[_XOPEN_PATH_MAX];
-
-        (void)strncpy(base, path, sizeof(base));
-        base[sizeof(base)-1] = 0;
-#ifndef _MSC_VER
-        (void)memmove(base, dirname(base), sizeof(base));
+        char base[_XOPEN_PATH_MAX];
+#ifdef _MSC_VER
+        {
+            char drive[_MAX_DRIVE+1]; // Will have trailing colon
+            char directory[_MAX_DIR+1]; // Will have trailing backslash
+            _splitpath(path, drive, directory, NULL, NULL);
+            (void)snprintf(base, sizeof(base), "%s%s", drive, directory);
+            base[sizeof(base)-1] = 0;
+        }
 #else
-		{
-			char *m_dir = (char*)malloc(sizeof(char)*1024);
-			_splitpath(base,NULL,m_dir,NULL,NULL);
-			(void)memmove(base,m_dir,sizeof(base));
-			free(m_dir);
-		}
+        {
+            // Temporary buffer used because `dirname()` modifies its argument
+            char tmp[strlen(path)+1];
+            (void)strcpy(tmp, path);
+            (void)strncpy(base, dirname(tmp), sizeof(base));
+            base[sizeof(base)-1] = 0;
+        }
 #endif
-		base[sizeof(base)-1] = 0;
 
         if (XML_SetBase(parser, base) != XML_STATUS_OK) {
             status = UT_OS;
