@@ -47,7 +47,7 @@ static const char*      _cmdWant; /* command-line "want" unit specification */
 static int		_reveal; /* reveal problems with unit database? */
 static int		_encodingSet; /* is the character encoding set? */
 static ut_encoding	_encoding; /* the character encoding to use */
-static const char*	_progname;
+static char             _progname[1024];
 static const char*	_xmlPath = NULL; /* use default path */
 static ut_system*	_unitSystem;
 static double           _haveUnitAmount; /* amount of "have" unit */
@@ -109,31 +109,37 @@ errMsg(
 
 static int
 decodeCommandLine(
-    int                 argc,
-    char* const*        argv)
+    int         argc,
+    char**      argv)
 {
     int		c;
     int		success = 0;
 
 #ifndef _MSC_VER
-	    _progname = basename(argv[0]);
- #else
-		{
-			char *m_fname = (char*)malloc(sizeof(char)*256);
-			char *m_ext = (char*)malloc(sizeof(char)*256);
-			char tmp[1024];
-			_splitpath(argv[0],NULL,NULL,m_fname,m_ext);
-			sprintf(tmp,"%s.%s",m_fname,m_ext);
-			_progname=tmp;
-			free(m_fname);
-			free(m_ext);
+    char* filename = basename(argv[0]);
 
-		}
+    if (strlen(filename)+1 > sizeof(_progname))
+        filename = "udunits2";
+
+    (void)strcpy(_progname, filename);
+#else
+    {
+        size_t len = strlen(argv[0]);
+        char*  m_fname = (char*)malloc(sizeof(char)*(len+1));
+        char*  m_ext = (char*)malloc(sizeof(char)*(len+1));
+
+        _splitpath(argv[0], NULL, NULL, m_fname, m_ext);
+
+        if (strlen(m_fname)+1 > sizeof(_progname))
+            (void)strcpy(_progname, "udunits2");
+        else
+            (void)strcpy(_progname, m_fname);
+
+        free(m_fname);
+        free(m_ext);
+    }
 #endif
 
-
-
-
     while ((c = getopt(argc, argv, "ALUhrH:W:")) != -1) {
 	switch (c) {
 	    case 'A':
@@ -390,7 +396,7 @@ getSpec(
 
     if (fputs(prompt, stdout) == EOF) {
 	errMsg("Couldn't write prompt: %s", strerror(errno));
-    } else if (fgets(spec, size, stdin) == NULL) {
+    } else if (fgets(spec, (int)size, stdin) == NULL) {
 	putchar('\n');
 
 	if (feof(stdin)) {
@@ -404,7 +410,7 @@ getSpec(
 	 */
 	(void)ut_trim(spec, _encoding);
 
-        nbytes = strlen(spec);
+        nbytes = (int)strlen(spec);
     }
 
     return nbytes;
@@ -413,29 +419,28 @@ getSpec(
 
 static int
 decodeInput(
-    const char* const   input)
+    const char* input)
 {
     int	success = 0;
 
     ut_free(_haveUnit);
 
-    if (sscanf(input, "%lg %[^ \t\n]", &_haveUnitAmount, _haveUnitSpec) == 2) {
-        _haveUnit = ut_parse(_unitSystem, _haveUnitSpec, _encoding);
-        if (_haveUnit != NULL)
-            success = 1;
+    int nbytes;
+    if (sscanf(input, "%lg %n", &_haveUnitAmount, &nbytes) == 1) {
+        input += nbytes;
+    }
+    else {
+        _haveUnitAmount = 1;
     }
 
-    if (!success) {
-        _haveUnitAmount = 1;
-        (void)strncpy(_haveUnitSpec, input, sizeof(_haveUnitSpec));
-        _haveUnitSpec[sizeof(_haveUnitSpec)-1] = 0;
-        _haveUnit = ut_parse(_unitSystem, _haveUnitSpec, _encoding);
-        if (_haveUnit == NULL) {
-            errMsg("Don't recognize \"%s\"", _haveUnitSpec);
-        }
-        else {
-            success = 1;
-        }
+    (void)strncpy(_haveUnitSpec, input, sizeof(_haveUnitSpec));
+    _haveUnitSpec[sizeof(_haveUnitSpec)-1] = 0;
+    _haveUnit = ut_parse(_unitSystem, _haveUnitSpec, _encoding);
+    if (_haveUnit == NULL) {
+        errMsg("Don't recognize \"%s\"", _haveUnitSpec);
+    }
+    else {
+        success = 1;
     }
 
     return success;
